; Written by: Fibergeek, http://www.codegurus.be/
; Date:       2004-10-03
; Subject:    Testing RIP relative addressing
;
; Updates:
;   2006-10-22
;     Added "public Main", this is needed from now on
;     Updated the command line you need to run
;     Removed the extra demonstration about absolute addressing, it's irrelevant with the newer ML64 versions
;
;   2005-09-12
;     The "END main" was turned into a simple "END"
;
; Compile & Link: ml64 rip.asm /link /subsystem:windows /defaultlib:kernel32.lib /defaultlib:user32.lib /entry:Main

extrn VirtualProtect: PROC
extrn MessageBoxA: PROC
extrn ExitProcess: PROC

.data
testok    db 'RIP based addressing worked, we executed "OR EAX, ECX" instead of "OR EAX, EAX"!', 0
testfail  db 'RIP based addressing didn''t work, we executed "OR EAX, EAX" instead of "OR EAX, ECX"!', 0
testtitle db 'RIP based addressing test by Fibergeek', 0

oldprotect dd ?

.code

public Main

Main:
  ; We first need to make this page writeable
  lea r9, oldprotect        ; R9  = lpflOldProtect
  mov r8d, 40h              ; R8D = flNewProtect (0x40 = PAGE_EXECUTE_READWRITE)
  mov rdx, 1                ; RDX = dwSize (NOTE: we will modify only one byte, but the whole page gets changed :))
                            ;              (this parameter is of type SIZE_T so it's 64-bits wide!)
  lea rcx, InXorInstruction ; RCX = lpAddress
  call VirtualProtect
  
  ; Ensure that the used registers for the test are setup
  mov eax, 0
  mov ecx, -1

  ; This is the test, we will increment C0 with one thus turning it into C1
  ; OR EAX, EAX will become OR EAX, ECX
  ;inc byte ptr [rip + 1] ; modify the next instruction - ML64 doesn't accept RIP as a parameter :((
  ;                         but will convert the address reference using RIP based addressing for us
  ;                         (this is because 64-bit mode doesn't support absolute addressing!?)
  inc InXorInstruction
XorInstruction   DB 00Bh ; 0BC0 = or eax, eax
InXorInstruction DB 0C0h

  ; Store in RAX the correct display text
  ; If EAX is 0 than the test failed!
  or eax, eax
  lea rax, testfail
  jz skip
  lea rax, testok
skip:

  ; Display the result and exit the program returning the result of the MessageBox
  mov r9d, 0         ; R9D = UINT uType
  lea r8,  testtitle ; R8  = LPCTSTR lpCaption
  mov rdx, rax       ; RDX = LPCSTR lpText
  mov rcx, 0         ; RCX = HWND hWnd (0 = HWND_DESKTOP)
  call MessageBoxA
  mov ecx, eax       ; ECX = UINT uExitCode
  call ExitProcess

  ; Just in case :)
  ret
END
